;;;-*-Mode: LISP; Package: CCL -*-
;;;
;;;   Copyright (C) 2002 Clozure Associates
;;;   This file is part of Opensourced MCL.
;;;
;;;   Opensourced MCL is free software; you can redistribute it and/or
;;;   modify it under the terms of the GNU Lesser General Public
;;;   License as published by the Free Software Foundation; either
;;;   version 2.1 of the License, or (at your option) any later version.
;;;
;;;   Opensourced MCL is distributed in the hope that it will be useful,
;;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;;   Lesser General Public License for more details.
;;;
;;;   You should have received a copy of the GNU Lesser General Public
;;;   License along with this library; if not, write to the Free Software
;;;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;;;

(in-package "CCL")			; for now.

(eval-when (:compile-toplevel :execute)
  (use-interface-dir :cocoa))

(eval-when (:compile-toplevel :load-toplevel :execute)
  (require "APPLE-OBJC"))



(eval-when (:compile-toplevel :execute)
  (setq *readtable* *objc-readtable*))
  


;;; Creating an "autorelease pool" soon after program startup is a
;;; common idiom; doing so helps the runtime system keep track of
;;; reference-counted objects.
(defun create-autorelease-pool ()
  [[(@class "NSAutoreleasePool") "alloc"] "init"])


(defun init-cocoa-application (&optional (bundle-path *default-bundle-path*))
  (when bundle-path (bash-argv bundle-path))
  (let* ((pool (create-autorelease-pool))
	 (bundle [(@class "NSBundle") "mainBundle"])
	 (dict [bundle "infoDictionary"])
	 (classname [dict "objectForKey:" :id (@"NSPrincipalClass")])
	 (mainnibname [dict "objectForKey:" :id (@"NSMainNibFile")]))
    (if (%null-ptr-p classname)
      (error "problems loading bundle: can't determine class name"))
    (if (%null-ptr-p mainnibname)
      (error "problems loading bundle: can't determine main nib name"))
    (let* ((appclass (external-call "_NSClassFromString"
				    :address classname
				    :address))
	   (app [ appclass "sharedApplication"]))
      [(@class "NSBundle") "loadNibNamed:owner:"
       :address mainnibname
       :address app]
      [pool "release"]
      app)))


(defun trace-dps-events (flag)
  (external-call "__DPSSetEventsTraced"
		 :unsigned-byte (if flag #$YES #$NO)
		 :void))


(def-objc-class "lispapplication" "NSApplication")

(defloadvar *distant-past* [(@class "NSDate") "distantPast"])
(defloadvar *distant-future* [(@class "NSDate") "distantPast"])
(defparameter *cocoa-event-poll-delay* 0.05d0)

(define-objc-method  ("nextEventMatchingMask:untilDate:inMode:dequeue:"
		      "lispapplication")
    (:unsigned mask :id date :id mode :<BOOL> dequeue :id)
  (let* ((date-not-in-future
	  (<= (the double-float
		[date "timeIntervalSinceNow" :double-float])
	      0.0d0))
	 (soon [(@class "NSDate") "dateWithTimeIntervalSinceNow:"
		:double-float 0.02d0]))
    (rlet ((eventptr (* t)))
      (flet ((cocoa-event-poll ()
	       (progn
		 (%setf-macptr
		  eventptr
		  [:super "nextEventMatchingMask:untilDate:inMode:dequeue:"
			  :unsigned mask
			  :id soon
			  :id mode
			  :<BOOL> dequeue])
		 (or (not (%null-ptr-p eventptr))
		     date-not-in-future))))
	(declare (dynamic-extent #'cocoa-event-poll))
	(process-wait "Cocoa Event Poll" #'cocoa-event-poll))
      eventptr)))



(defloadvar *NSApp* nil )
(defloadvar *default-ns-application-proxy-class-name*
    "lispapplicationdelegate")

(defun start-cocoa-application (&key
				(bundle-path *default-bundle-path*)
				(application-proxy-class-name
				 *default-ns-application-proxy-class-name*)
				(event-process t))
  (or *NSApp* (setq *NSApp* (init-cocoa-application bundle-path)))
  (when application-proxy-class-name
    (let* ((classptr (%objc-class-classptr
		      (load-objc-class-descriptor application-proxy-class-name))))
      [*NSApp* "setDelegate:" :address [[classptr "alloc"]
				  "init"] :void]))
  (flet ((run () [*NSApp* "run"]))
    (if event-process
      (process-run-function
       (list :name "Cocoa Event Loop" :stack-size (ash 1 18))
       #'run)
      (run))))
